@@ -54,8 +54,16 @@
 #define DEFAULT_TIMEOUT 5000
 #endif
 
+char *request;
+char *response;
+char *received;
+
 char *usern;
 
+int inv_trans;
+
+char fqdn[FQDN_SIZE];
+
 enum usteps usrlocstep;
 
 struct sipsak_regexp regexps;
@@ -65,11 +73,13 @@ struct sipsak_con_data cdata;
 struct sipsak_counter counters;
 struct sipsak_delay delays;
 
+struct sipsak_msg_data msg_data;
+
 /* if a reply was received successfully, return success, unless
  * reply matching is enabled and no match occurred
  */
 
-static inline void on_success(char *_response)
+static inline void on_success(char *_response, regex_t *regex)
 {
 	if ((_response != NULL) && regex &&
 			regexec(regex, _response, 0, 0, 0) == REG_NOMATCH) {
@@ -94,7 +104,13 @@ static inline void create_usern(char *target, char *username, int number)
 }
 
 /* tries to take care of a redirection */
-void handle_3xx(struct sockaddr_in *tadr)
+void handle_3xx(struct sockaddr_in *tadr, int warning_ext, int rport,
+    unsigned long address, unsigned int transport, int outbound_proxy,
+    char *domainname
+#ifdef WITH_TLS_TRANSP
+    , int ignore_ca_fail
+#endif
+    )
 {
 	char *uscheme, *uuser, *uhost, *contact;
 
@@ -119,7 +135,7 @@ void handle_3xx(struct sockaddr_in *tadr)
 		}
 		/* correct our request */
 		uri_replace(request, contact);
-		new_transaction(request, response);
+		msg_data.cseq_counter = new_transaction(request, response);
 		/* extract the needed information*/
 		rport = 0;
 		address = 0;
@@ -139,7 +155,12 @@ void handle_3xx(struct sockaddr_in *tadr)
 		}
 		free(contact);
 		if (!outbound_proxy)
-			cdata.connected = set_target(tadr, address, rport, cdata.csock, cdata.connected);
+			cdata.connected = set_target(tadr, address, rport, cdata.csock,
+          cdata.connected, cdata.transport, domainname
+#ifdef WITH_TLS_TRANSP
+          , ignore_ca_fail
+#endif
+          );
 	}
 	else {
 		fprintf(stderr, "error: cannot handle this redirect:"
@@ -149,7 +170,7 @@ void handle_3xx(struct sockaddr_in *tadr)
 }
 
 /* takes care of replies in the trace route mode */
-void trace_reply()
+void trace_reply(regex_t *regex, int namebeg, struct sipsak_sr_time *timers)
 {
 	char *contact;
 
@@ -158,16 +179,17 @@ void trace_reply()
 		printf("%i: ", namebeg);
 		if (verbose > 2) {
 			printf("(%.3f ms)\n%s\n", 
-				deltaT(&(timers.sendtime), &(timers.recvtime)), received);
+				deltaT(&(timers->sendtime), &(timers->recvtime)), received);
 		}
 		else {
 			warning_extract(received);
-			printf("(%.3f ms) ", deltaT(&(timers.sendtime), &(timers.recvtime)));
+			printf("(%.3f ms) ", deltaT(&(timers->sendtime), &(timers->recvtime)));
 			print_message_line(received);
 		}
+    // FIXME looks like we want to modify the global value here
 		namebeg++;
-		cseq_counter++;
-		create_msg(REQ_OPT, request, NULL, usern, cseq_counter);
+		msg_data.cseq_counter++;
+		create_msg(REQ_OPT, &msg_data);
 		set_maxforw(request, namebeg);
 		return;
 	}
@@ -176,14 +198,14 @@ void trace_reply()
 		printf("%i: ", namebeg);
 		if (verbose > 2) {
 			printf("(%.3f ms)\n%s\n", 
-				deltaT(&(timers.sendtime), &(timers.recvtime)), received);
+				deltaT(&(timers->sendtime), &(timers->recvtime)), received);
 		}
 		else {
 			warning_extract(received);
-			printf("(%.3f ms) ", deltaT(&(timers.sendtime), &(timers.recvtime)));
+			printf("(%.3f ms) ", deltaT(&(timers->sendtime), &(timers->recvtime)));
 			print_message_line(received);
 		}
-		delays.retryAfter = timer_t2;
+		delays.retryAfter = timers->timer_t2;
 		cdata.dontsend=1;
 		return;
 	}
@@ -192,7 +214,7 @@ void trace_reply()
 		   be treated as final */
 		printf("%i: ", namebeg);
 		warning_extract(received);
-		printf("(%.3f ms) ", deltaT(&(timers.sendtime), &(timers.recvtime)));
+		printf("(%.3f ms) ", deltaT(&(timers->sendtime), &(timers->recvtime)));
 		print_message_line(received);
 		if ((contact = STRCASESTR(received, CONT_STR)) != NULL ||
 				(contact = STRCASESTR(received, CONT_SHORT_STR)) != NULL) {
@@ -206,7 +228,7 @@ void trace_reply()
 			printf("\twithout Contact header\n");
 		}
 		if (regexec(&(regexps.okexp), received, 0, 0, 0) == REG_NOERROR) {
-			on_success(received);
+			on_success(received, regex);
 		} else {
 			log_message(request);
 			exit_code(1, __PRETTY_FUNCTION__, "received final non-2xx reply");
@@ -215,7 +237,7 @@ void trace_reply()
 }
 
 /* takes care of replies in the default mode */
-void handle_default()
+void handle_default(regex_t *regex, struct sipsak_sr_time *timers)
 {
 	/* in the normal send and reply case anything other 
 	   then 1xx will be treated as final response*/
@@ -224,12 +246,12 @@ void handle_default()
 			printf("%s\n\n", received);
 			printf("** reply received ");
 			if ((counters.send_counter == 1) || (STRNCASECMP(request, ACK_STR, ACK_STR_LEN) == 0)) {
-				printf("after %.3f ms **\n", deltaT(&(timers.firstsendt), &(timers.recvtime)));
+				printf("after %.3f ms **\n", deltaT(&(timers->firstsendt), &(timers->recvtime)));
 			}
 			else {
 				printf("%.3f ms after first send\n   and "
-						"%.3f ms after last send **\n", deltaT(&(timers.firstsendt), &(timers.recvtime)), 
-						deltaT(&(timers.sendtime), &(timers.recvtime)));
+						"%.3f ms after last send **\n", deltaT(&(timers->firstsendt),
+              &(timers->recvtime)), deltaT(&(timers->sendtime), &(timers->recvtime)));
 			}
 			printf("   ");
 			print_message_line(received);
@@ -237,10 +259,10 @@ void handle_default()
 					" waiting for a final response\n");
 		}
 		if (inv_trans) {
-			delays.retryAfter = timer_final;
+			delays.retryAfter = timers->timer_final;
 		}
 		else {
-			delays.retryAfter = timer_t2;
+			delays.retryAfter = timers->timer_t2;
 		}
 		cdata.dontsend = 1;
 		return;
@@ -250,12 +272,12 @@ void handle_default()
 			printf("%s\n\n", received);
 			printf("** reply received ");
 			if ((counters.send_counter == 1) || (STRNCASECMP(request, ACK_STR, ACK_STR_LEN) == 0)){
-				printf("after %.3f ms **\n", deltaT(&(timers.firstsendt), &(timers.recvtime)));
+				printf("after %.3f ms **\n", deltaT(&(timers->firstsendt), &(timers->recvtime)));
 			}
 			else {
 				printf("%.3f ms after first send\n   and "
-						"%.3f ms after last send **\n", deltaT(&(timers.firstsendt), &(timers.recvtime)), 
-						deltaT(&(timers.sendtime), &(timers.recvtime)));
+						"%.3f ms after last send **\n", deltaT(&(timers->firstsendt),
+              &(timers->recvtime)), deltaT(&(timers->sendtime), &(timers->recvtime)));
 			}
 			printf("   ");
 			print_message_line(received);
@@ -264,9 +286,9 @@ void handle_default()
 		else if (verbose>0) {
 			printf("%s\n", received);
 		}
-		if (timing > 0) {
-			timing--;
-			if (timing == 0) {
+		if (timers->timing > 0) {
+			timers->timing--;
+			if (timers->timing == 0) {
 				if (counters.run == 0) {
 					counters.run++;
 				}
@@ -274,13 +296,13 @@ void handle_default()
 			}
 			else {
 				counters.run++;
-				new_transaction(request, response);
-				delays.retryAfter = timer_t1;
+				msg_data.cseq_counter = new_transaction(request, response);
+				delays.retryAfter = timers->timer_t1;
 			}
 		}
-		if (timing == 0) {
+		if (timers->timing == 0) {
 			if (regexec(&(regexps.okexp), received, 0, 0, 0) == REG_NOERROR) {
-				on_success(received);
+				on_success(received, regex);
 			}
 			else {
 				log_message(request);
@@ -291,7 +313,7 @@ void handle_default()
 }
 
 /* takes care of replies in the readntrash mode */
-void handle_randtrash()
+void handle_randtrash(int warning_ext, int nameend)
 {
 	/* in randomzing trash we are expexting 4?? error codes
 	   everything else should not be normal */
@@ -315,7 +337,7 @@ void handle_randtrash()
 		if (verbose > 1) 
 			printf("sended:\n%s\nreceived:\n%s\n", request, received);
 	}
-	if (cseq_counter == nameend) {
+	if (msg_data.cseq_counter == nameend) {
 		if (counters.randretrys == 0) {
 			printf("random end reached. server survived :) respect!\n");
 			exit_code(0, __PRETTY_FUNCTION__, NULL);
@@ -333,7 +355,9 @@ void handle_randtrash()
 }
 
 /* takes care of replies in the usrloc mode */
-void handle_usrloc()
+void handle_usrloc(regex_t *regex, int namebeg, int nameend, int rand_rem,
+    char *username, int nagios_warn, struct sipsak_sr_time *timers,
+    char *mes_body, enum sipsak_modes mode)
 {
 	char *crlf;
 	char ruri[11+12+20]; //FIXME: username length 20 should be dynamic
@@ -344,10 +368,10 @@ void handle_usrloc()
 			printf("ignoring provisional response\n\n");
 		}
 		if (inv_trans) {
-			delays.retryAfter = timer_final;
+			delays.retryAfter = timers->timer_final;
 		}
 		else {
-			delays.retryAfter = timer_t2;
+			delays.retryAfter = timers->timer_t2;
 		}
 		cdata.dontsend = 1;
 	}
@@ -371,14 +395,14 @@ void handle_usrloc()
 					log_message(request);
 					exit_code(1, __PRETTY_FUNCTION__, "received non-2xx reply for REGISTER");
 				}
-				if (invite == 0 && message == 0) {
+				if (mode == SM_USRLOC) {
 					if (namebeg==nameend) {
 						if (verbose>0)  {
 							printf("\nAll usrloc tests"
 										" completed successful.\nreceived"
 										" last message %.3f ms after first"
 										" request (test duration).\n", 
-										deltaT(&(timers.firstsendt), &(timers.recvtime)));
+										deltaT(&(timers->firstsendt), &(timers->recvtime)));
 						}
 						if (delays.big_delay>0 && verbose>0) {
 							printf("biggest delay between "
@@ -399,38 +423,39 @@ void handle_usrloc()
 								exit_code(4, __PRETTY_FUNCTION__, "#retransmissions above nagios warn level");
 							}
 						}
-						if (timing) {
+						if (timers->timing) {
 							printf("%.3f ms\n",
-										deltaT(&(timers.firstsendt), &(timers.recvtime)));
+										deltaT(&(timers->firstsendt), &(timers->recvtime)));
 						}
-						on_success(received);
+						on_success(received, regex);
 					} /* namebeg == nameend */
 					/* lets see if we deceid to remove a 
 					   binding (case 6)*/
 					if ( ((float)rand()/RAND_MAX)*100 > rand_rem) {
+            // FIXME we want to modify the global value here
 						namebeg++;
-						cseq_counter++;
+						msg_data.cseq_counter++;
 						create_usern(usern, username, namebeg);
-						create_msg(REQ_REG, request, NULL, usern, cseq_counter);
+						create_msg(REQ_REG, &msg_data);
 					}
 					else {
 						/* to prevent only removing of low
 						   user numbers new random number*/
-						cseq_counter++;
+						msg_data.cseq_counter++;
 						create_usern(usern, username, ((float)rand()/RAND_MAX) * namebeg);
-						create_msg(REQ_REM, request, NULL, usern, cseq_counter);
+						create_msg(REQ_REM, &msg_data);
 						usrlocstep=UNREG_REP;
 					}
 				} /* invite == 0 && message == 0 */
-				else if (invite == 1) {
-					cseq_counter++;
-					create_msg(REQ_INV, request, response, usern, cseq_counter);
+				else if (mode == SM_USRLOC_INVITE) {
+					msg_data.cseq_counter++;
+					create_msg(REQ_INV, &msg_data);
 					inv_trans = 1;
 					usrlocstep=INV_RECV;
 				}
-				else if (message == 1) {
-					cseq_counter++;
-					create_msg(REQ_MES, request, response, usern, cseq_counter);
+				else if (mode == SM_USRLOC_MESSAGE) {
+					msg_data.cseq_counter++;
+					create_msg(REQ_MES, &msg_data);
 					inv_trans = 0;
 					usrlocstep=MES_RECV;
 				}
@@ -482,11 +507,11 @@ void handle_usrloc()
 				}
 				else {
 					fprintf(stderr, "received:\n%s\nerror: did not "
-								"received the '200 OK' that was sent "
+								"receive the '200 OK' that was sent "
 								"as the reply on the INVITE (see "
 								"above). aborting\n", received);
 					log_message(request);
-					exit_code(1, __PRETTY_FUNCTION__, "did not received our own 200 reply");
+					exit_code(1, __PRETTY_FUNCTION__, "did not receive our own 200 reply");
 				}
 				break;
 			case INV_ACK_RECV:
@@ -519,7 +544,8 @@ void handle_usrloc()
 							printf("\nAll usrloc tests completed "
 										"successful.\nreceived last message"
 										" %.3f ms after first request (test"
-										" duration).\n", deltaT(&(timers.firstsendt), &(timers.recvtime)));
+										" duration).\n", deltaT(&(timers->firstsendt),
+                      &(timers->recvtime)));
 						}
 						if (delays.big_delay>0) {
 							printf("biggest delay between "
@@ -540,43 +566,45 @@ void handle_usrloc()
 								exit_code(4, __PRETTY_FUNCTION__, "#retransmissions above nagios warn level");
 							}
 						}
-						on_success(received);
+						on_success(received, regex);
 					} /* namebeg == nameend */
-					if (usrloc == 1) {
+					if (mode == SM_USRLOC_INVITE) {
 						/* lets see if we deceid to remove a 
 						   binding (case 6)*/
 						if (((float)rand()/RAND_MAX) * 100 > rand_rem) {
+              // FIXME we want to modify the global value here
 							namebeg++;
-							cseq_counter++;
+							msg_data.cseq_counter++;
 							create_usern(usern, username, namebeg);
-							create_msg(REQ_REG, request, NULL, usern, cseq_counter);
+							create_msg(REQ_REG, &msg_data);
 							usrlocstep=REG_REP;
 						}
 						else {
 							/* to prevent only removing of low
 							   user numbers new random number*/
-							cseq_counter++;
+							msg_data.cseq_counter++;
 							create_usern(usern, username, ((float)rand()/RAND_MAX) * namebeg);
-							create_msg(REQ_REM, request, NULL, usern, cseq_counter);
+							create_msg(REQ_REM, &msg_data);
 							usrlocstep=UNREG_REP;
 						}
 					} /* usrloc == 1 */
 					else {
+              // FIXME we want to modify the global value here
 						namebeg++;
-						cseq_counter++;
+						msg_data.cseq_counter++;
 						create_usern(usern, username, namebeg);
-						create_msg(REQ_INV, request, response, usern, cseq_counter);
+						create_msg(REQ_INV, &msg_data);
 						inv_trans = 1;
 						usrlocstep=INV_RECV;
 					}
 				} /* STRNCASECMP */
 				else {
 					fprintf(stderr, "received:\n%s\nerror: did not "
-								"received the 'ACK' that was sent "
+								"receive the 'ACK' that was sent "
 								"as the reply on the '200 OK' (see "
 								"above). aborting\n", received);
 					log_message(request);
-					exit_code(1, __PRETTY_FUNCTION__, "missing ACK that was send by myself");
+					exit_code(1, __PRETTY_FUNCTION__, "missing ACK that was sent by myself");
 				}
 				break;
 			case MES_RECV:
@@ -598,10 +626,10 @@ void handle_usrloc()
 				}
 				else {
 					fprintf(stderr, "received:\n%s\nerror: did not "
-								"received the 'MESSAGE' that was sent "
+								"receive the 'MESSAGE' that was sent "
 								"(see above). aborting\n", received);
 					log_message(request);
-					exit_code(1, __PRETTY_FUNCTION__, "did not received my own MESSAGE request");
+					exit_code(1, __PRETTY_FUNCTION__, "did not receive my own MESSAGE request");
 				}
 				break;
 			case MES_OK_RECV:
@@ -631,7 +659,8 @@ void handle_usrloc()
 							printf("\nAll usrloc tests completed "
 										"successful.\nreceived last message"
 										" %.3f ms after first request (test"
-										" duration).\n", deltaT(&(timers.firstsendt), &(timers.recvtime)));
+										" duration).\n", deltaT(&(timers->firstsendt),
+                      &(timers->recvtime)));
 						}
 						if (delays.big_delay>0) {
 							printf("biggest delay between "
@@ -653,32 +682,34 @@ void handle_usrloc()
 								exit_code(4, __PRETTY_FUNCTION__, "#retransmissions above nagios warn level");
 							}
 						}
-						on_success(received);
+						on_success(received, regex);
 					} /* namebeg == nameend */
-					if (usrloc == 1) {
+					if (mode == SM_USRLOC_MESSAGE) {
 						/* lets see if we deceid to remove a 
 						   binding (case 6)*/
 						if (((float)rand()/RAND_MAX) * 100 > rand_rem) {
+              // FIXME we want to modify the global value here
 							namebeg++;
-							cseq_counter++;
+							msg_data.cseq_counter++;
 							create_usern(usern, username, namebeg);
-							create_msg(REQ_REG, request, NULL, usern, cseq_counter);
+							create_msg(REQ_REG, &msg_data);
 							usrlocstep=REG_REP;
 						}
 						else {
 							/* to prevent only removing of low
 							   user numbers new random number*/
-							cseq_counter++;
+							msg_data.cseq_counter++;
 							create_usern(usern, username, ((float)rand()/RAND_MAX) * namebeg);
-							create_msg(REQ_REM, request, NULL, usern, cseq_counter);
+							create_msg(REQ_REM, &msg_data);
 							usrlocstep=UNREG_REP;
 						}
 					} /* usrloc == 1 */
 					else {
+              // FIXME we want to modify the global value here
 						namebeg++;
-						cseq_counter++;
+						msg_data.cseq_counter++;
 						create_usern(usern, username, namebeg);
-						create_msg(REQ_MES, request, NULL, usern, cseq_counter);
+						create_msg(REQ_MES, &msg_data);
 						usrlocstep=MES_RECV;
 					}
 				} /* regexec */
@@ -686,13 +717,13 @@ void handle_usrloc()
 					if (verbose>0) {
 						if (mes_body) {
 							fprintf(stderr, "received:\n%s\nerror: did"
-										" not received 200 for the "
+										" not receive 200 for the "
 										"MESSAGE (see above)\n",
 										received);
 						}
 						else {
 							fprintf(stderr, "received:\n%s\nerror: did"
-										" not received the '200 OK' "
+										" not receive the '200 OK' "
 										"that was sent as the reply on"
 										" the MESSAGE (see above). "
 										"aborting\n", received);
@@ -722,15 +753,16 @@ void handle_usrloc()
 					else if (verbose>0) {
 						printf("Binding removal for %s successful\n", username);
 					}
+              // FIXME we want to modify the global value here
 					namebeg++;
-					cseq_counter++;
+					msg_data.cseq_counter++;
 					create_usern(usern, username, namebeg);
-					create_msg(REQ_REG, request, NULL, usern, cseq_counter);
+					create_msg(REQ_REG, &msg_data);
 					usrlocstep=REG_REP;
 				}
 				else {
 					fprintf(stderr, "received:\n%s\nerror: did not "
-								"received the expected 200 on the "
+								"receive the expected 200 on the "
 								"remove bindings request for %s%i (see"
 								" above). aborting\n", received, username, 
 								namebeg);
@@ -746,22 +778,29 @@ void handle_usrloc()
 	} /* regexec proexp */
 }
 
-void before_sending()
+void before_sending(struct sipsak_counter *counter, struct sipsak_msg_data *msg_data,
+    enum sipsak_modes mode)
 {
 	/* some initial output */
-	if ((usrloc == 1||invite == 1||message == 1) && (verbose > 1) && (cdata.dontsend == 0)) {
+	if ((mode == SM_USRLOC ||
+       mode == SM_USRLOC_INVITE ||
+       mode == SM_USRLOC_MESSAGE ||
+       mode == SM_INVITE ||
+       mode == SM_MESSAGE) &&
+      (verbose > 1) &&
+      (cdata.dontsend == 0)) {
 		switch (usrlocstep) {
 			case REG_REP:
-				if (nameend>0)
-					printf("registering user %s%i... ", username, namebeg);
+				if (counter->nameend>0)
+					printf("registering user %s%i... ", msg_data->username, counter->namebeg);
 				else
-					printf("registering user %s... ", username);
+					printf("registering user %s... ", msg_data->username);
 				break;
 			case INV_RECV:
-				if (nameend>0)
-					printf("inviting user %s%i... ", username, namebeg);
+				if (counter->nameend>0)
+					printf("inviting user %s%i... ", msg_data->username, counter->namebeg);
 				else
-					printf("inviting user %s... ", username);
+					printf("inviting user %s... ", msg_data->username);
 				break;
 			case INV_OK_RECV:
 				printf("sending invite reply... ");
@@ -770,66 +809,69 @@ void before_sending()
 				printf("sending invite ack... ");
 				break;
 			case MES_RECV:
-				if (nameend>0)
-					printf("sending message to %s%i... ", username, namebeg);
+				if (counter->nameend>0)
+					printf("sending message to %s%i... ", msg_data->username, counter->namebeg);
 				else
-					printf("sending message to %s... ", username);
+					printf("sending message to %s... ", msg_data->username);
 				break;
 			case MES_OK_RECV:
-				if (mes_body)
+				if (msg_data->mes_body)
 					printf("sending message ... \n");
 				else
 					printf("sending message reply... ");
 				break;
 			case UNREG_REP:
-				if (nameend>0)
-					printf("remove binding for %s%i...", username, namebeg);
+				if (counter->nameend>0)
+					printf("remove binding for %s%i...", msg_data->username, counter->namebeg);
 				else
-					printf("remove binding for %s...", username);
+					printf("remove binding for %s...", msg_data->username);
 				break;
 		}
 	} /* if usrloc...*/
-	else if (flood == 1 && verbose > 0) {
-		printf("flooding message number %i\n", namebeg);
+	else if (mode == SM_FLOOD &&
+           verbose > 0) {
+		printf("flooding message number %i\n", counter->namebeg);
 	}
-	else if (randtrash == 1 && verbose > 0) {
-		printf("message with %i randomized chars\n", cseq_counter);
+	else if (mode == SM_RANDTRASH &&
+      verbose > 0) {
+		printf("message with %i randomized chars\n", msg_data->cseq_counter);
 		if (verbose > 2)
 			printf("request:\n%s\n", request);
 	}
 }
 
 /* this is the main function with the loops and modes */
-void shoot(char *buf, int buff_size)
+void shoot(char *buf, int buff_size, struct sipsak_options *options)
 {
 	struct timespec sleep_ms_s, sleep_rem;
 	int ret, cseqtmp, rand_tmp;
 	char buf2[BUFSIZE], buf3[BUFSIZE], lport_str[LPORT_STR_LEN];
 
-	/* delays.retryAfter = DEFAULT_TIMEOUT; */
-	if (transport == SIP_UDP_TRANSPORT) {
-		delays.retryAfter = timer_t1;
-	}
-	else {
-		delays.retryAfter = timer_final;
-	}
 	inv_trans = 0;
-	cseq_counter = 1;
 	usrlocstep = REG_REP;
 
 	/* initalize local vars */
 	cdata.dontsend=cdata.dontrecv=counters.retrans_r_c=counters.retrans_s_c= 0;
 	delays.big_delay=counters.send_counter=counters.run= 0;
-	timers.delaytime.tv_sec = 0;
-	timers.delaytime.tv_usec = 0;
 	usern = NULL;
 	/* initialize local arrays */
 	memset(buf2, 0, BUFSIZE);
 	memset(buf3, 0, BUFSIZE);
 	memset(lport_str, 0, LPORT_STR_LEN);
+	memset(fqdn, 0, FQDN_SIZE);
+
+  /* initialize external vars which don't have initializer */
+  nonce_count = 0;
+
+	counters.namebeg = options->namebeg;
+	counters.nameend = options->nameend;
 
 	cdata.csock = cdata.usock = -1;
 	cdata.connected = 0;
+	cdata.transport = options->transport;
+	cdata.symmetric = options->symmetric;
+	cdata.lport = options->lport;
+	cdata.rport = options->rport;
 	cdata.buf_tmp = NULL;
 	cdata.buf_tmp_size = 0;
 
@@ -838,35 +880,58 @@ void shoot(char *buf, int buff_size)
 	memset(&(timers.firstsendt), 0, sizeof(timers.firstsendt));
 	memset(&(timers.starttime), 0, sizeof(timers.starttime));
 	memset(&(timers.delaytime), 0, sizeof(timers.delaytime));
+	timers.timer_t1 = options->timer_t1;
+	timers.timer_t2 = 8 * timers.timer_t1;
+	timers.timer_final = options->timer_final * timers.timer_t1;
+	timers.timing = options->timing;
+
+	/* delays.retryAfter = DEFAULT_TIMEOUT; */
+	if (cdata.transport == SIP_UDP_TRANSPORT) {
+		delays.retryAfter = timers.timer_t1;
+	}
+	else {
+		delays.retryAfter = timers.timer_final;
+	}
 
 	request = buf;
 	response = buf2;
 	received = buf3;
 
-	create_sockets(&cdata);
+	msg_data.cseq_counter = 1;
+	msg_data.lport = cdata.lport;
+	msg_data.expires_t = options->expires_t;
+	msg_data.empty_contact = options->empty_contact;
+	msg_data.transport = options->transport;
+	msg_data.req_buff = request;
+	msg_data.repl_buff = NULL;
+	msg_data.username = options->username;
+	msg_data.domainname = options->domainname;
+	msg_data.contact_uri = options->contact_uri;
+	msg_data.con_dis = options->con_dis;
+	msg_data.from_uri = options->from_uri;
+	msg_data.mes_body = options->mes_body;
+	msg_data.headers = options->headers;
+	msg_data.fqdn = fqdn;
 
-	if (sleep_ms != 0) {
-		if (sleep_ms == -2) {
-			rand_tmp = rand();
-			sleep_ms_s.tv_sec = rand_tmp / 1000;
-			sleep_ms_s.tv_nsec = (rand_tmp % 1000) * 1000000;
-		}
-		else {
-			sleep_ms_s.tv_sec = sleep_ms / 1000;
-			sleep_ms_s.tv_nsec = (sleep_ms % 1000) * 1000000;
-		}
-	}
+	init_network(&cdata, options->local_ip
+#ifdef WITH_TLS_TRANSP
+      , options->ca_file
+#endif
+      );
 
-	if (replace_b == 1){
-		replace_string(request, "$dsthost$", domainname);
+	/* determine our hostname */
+	get_fqdn(&fqdn[0], options->numeric, options->hostname);
+
+	if (options->replace_b == 1){
+		replace_string(request, "$dsthost$", options->domainname);
 		replace_string(request, "$srchost$", fqdn);
-		sprintf(lport_str, "%i", lport);
+		sprintf(lport_str, "%i", cdata.lport);
 		replace_string(request, "$port$", lport_str);
-		if (username)
-			replace_string(request, "$user$", username);
+		if (msg_data.username)
+			replace_string(request, "$user$", msg_data.username);
 	}
-	if (replace_str)
-		replace_strings(request, replace_str);
+	if (options->replace_str)
+		replace_strings(request, options->replace_str);
 
 	/* set all regular expression to simplfy the result code identification */
 	regcomp(&(regexps.replyexp), "^SIP/[0-9]\\.[0-9] [1-6][0-9][0-9]", 
@@ -884,60 +949,64 @@ void shoot(char *buf, int buff_size)
 	regcomp(&(regexps.tmhexp), "^SIP/[0-9]\\.[0-9] 483 ", 
 		REG_EXTENDED|REG_NOSUB|REG_ICASE); 
 
-	if (username) {
-		if (nameend > 0) {
-			usern = str_alloc(strlen(username) + 12);
-			create_usern(usern, username, namebeg);
+	if (msg_data.username) {
+		if (counters.nameend > 0) {
+			usern = str_alloc(strlen(msg_data.username) + 12);
+			create_usern(usern, msg_data.username, counters.namebeg);
+      msg_data.username = usern;
 		}
 		else {
-			if (*(username + strlen(username) - 1) != '@') {
-				usern = str_alloc(strlen(username) + 2);
-				create_usern(usern, username, -1);
+			if (*(msg_data.username + strlen(msg_data.username) - 1) != '@') {
+				usern = str_alloc(strlen(msg_data.username) + 2);
+				create_usern(usern, msg_data.username, -1);
+        msg_data.username = usern;
 			}
 			else {
-				usern = username;
+				usern = msg_data.username;
 			}
 		}
 	}
 
-	if (usrloc == 1||invite == 1||message == 1){
-		/* calculate the number of required steps and create initial mes */
-		if (usrloc == 1) {
-			create_msg(REQ_REG, request, NULL, usern, cseq_counter);
-			usrlocstep=REG_REP;
-		}
-		else if (invite == 1) {
-			create_msg(REQ_INV, request, response, usern, cseq_counter);
-			inv_trans = 1;
-			usrlocstep=INV_RECV;
-		}
-		else {
-			create_msg(REQ_MES, request, response, usern, cseq_counter);
-			if (mes_body)
-				usrlocstep=MES_OK_RECV;
-			else
-				usrlocstep=MES_RECV;
-		}
-	}
-	else if (trace == 1){
+	/* calculate the number of required steps and create initial mes */
+  if (options->mode == SM_USRLOC ||
+      options->mode == SM_USRLOC_INVITE ||
+      options->mode == SM_USRLOC_MESSAGE) {
+    create_msg(REQ_REG, &msg_data);
+    usrlocstep=REG_REP;
+  }
+  else if (options->mode == SM_INVITE) {
+    create_msg(REQ_INV, &msg_data);
+    inv_trans = 1;
+    usrlocstep=INV_RECV;
+  }
+  else if (options->mode == SM_MESSAGE) {
+    create_msg(REQ_MES, &msg_data);
+    if (msg_data.mes_body)
+      usrlocstep=MES_OK_RECV;
+    else
+      usrlocstep=MES_RECV;
+  }
+	else if (options->mode == SM_TRACE){
 		/* for trace we need some spezial initis */
-		namebeg=0;
-		create_msg(REQ_OPT, request, NULL, usern, cseq_counter);
-		set_maxforw(request, namebeg);
+		counters.namebeg=0;
+		create_msg(REQ_OPT, &msg_data);
+		set_maxforw(request, counters.namebeg);
 	}
-	else if (flood == 1){
-		if (nameend<=0) nameend=INT_MAX;
-		namebeg=1;
-		create_msg(REQ_FLOOD, request, NULL, usern, cseq_counter);
+	else if (options->mode == SM_FLOOD){
+		if (counters.nameend<=0) {
+      counters.nameend=INT_MAX;
+    }
+		counters.namebeg=1;
+		create_msg(REQ_FLOOD, &msg_data);
 	}
-	else if (randtrash == 1){
+	else if (options->mode == SM_RANDTRASH){
 		counters.randretrys=0;
-		namebeg=1;
-		create_msg(REQ_RAND, request, NULL, usern, cseq_counter);
-		nameend=(int)strlen(request);
-		if (trashchar == 1){
-			if (trashchar < nameend)
-				nameend=trashchar;
+		counters.namebeg=1;
+		create_msg(REQ_RAND, &msg_data);
+		counters.nameend=(int)strlen(request);
+		if (options->trashchar == 1){
+			if (options->trashchar < counters.nameend)
+				counters.nameend=options->trashchar;
 			else
 				fprintf(stderr, "warning: number of trashed chars to big. setting to "
 					"request length\n");
@@ -946,37 +1015,42 @@ void shoot(char *buf, int buff_size)
 	}
 	else {
 		/* for none of the modes we also need some inits */
-		if (file_b == 0) {
-			namebeg=1;
-			create_msg(REQ_OPT, request, NULL, usern, cseq_counter);
+		if (options->file_b == 0) {
+			counters.namebeg=1;
+			create_msg(REQ_OPT, &msg_data);
 		}
 		else {
 			if (STRNCASECMP(request, INV_STR, INV_STR_LEN) == 0) {
 				inv_trans = 1;
 			}
-			if(via_ins == 1)
-				add_via(request);
+			if(options->via_ins == 1)
+				add_via(request, msg_data.fqdn, msg_data.lport);
 		}
 		/* delays.retryAfter = delays.retryAfter / 10; */
-		if(maxforw!=-1)
-			set_maxforw(request, maxforw);
+		if(options->maxforw != -1)
+			set_maxforw(request, options->maxforw);
 	}
 
-	cdata.connected = set_target(&(cdata.adr), address, rport, cdata.csock, cdata.connected);
+	cdata.connected = set_target(&(cdata.adr), options->address, cdata.rport, cdata.csock,
+      cdata.connected, cdata.transport, options->domainname
+#ifdef WITH_TLS_TRANSP
+      , options->ignore_ca_fail
+#endif
+      );
 
 	/* here we go until someone decides to exit */
 	while(1) {
-		before_sending();
+		before_sending(&counters, &msg_data, options->mode);
 
-		if (sleep_ms == -2) {
+		if (options->sleep_ms == -2) {
 			rand_tmp = rand();
 			sleep_ms_s.tv_sec = rand_tmp / 1000;
 			sleep_ms_s.tv_nsec = (rand_tmp % 1000) * 1000;
-		} else if (sleep_ms != 0) {
-			sleep_ms_s.tv_sec = sleep_ms;
-			sleep_ms_s.tv_nsec = 0;
+		} else if (options->sleep_ms != 0) {
+			sleep_ms_s.tv_sec = options->sleep_ms;
+			sleep_ms_s.tv_nsec = (options->sleep_ms % 1000) * 1000000;
 		}
-		if (sleep_ms != 0) {
+		if (options->sleep_ms != 0) {
 			dbg("sleeping for %li s + %li ns\n", sleep_ms_s.tv_sec, sleep_ms_s.tv_nsec);
 			nanosleep(&sleep_ms_s, &sleep_rem);
 		}
@@ -984,9 +1058,10 @@ void shoot(char *buf, int buff_size)
 		send_message(request, &cdata, &counters, &timers);
 
 		/* in flood we are only interested in sending so skip the rest */
-		if (flood == 0) {
+		if (options->mode != SM_FLOOD) {
 			ret = recv_message(received, BUFSIZE, inv_trans, &delays, &timers,
-						&counters, &cdata, &regexps);
+						&counters, &cdata, &regexps, options->mode, msg_data.cseq_counter,
+            request, response);
 			if(ret > 0)
 			{
 				if (usrlocstep == INV_OK_RECV) {
@@ -1005,7 +1080,7 @@ void shoot(char *buf, int buff_size)
 				}
 				/* check for old CSeq => ignore retransmission */
 				cseqtmp = cseq(received);
-				if ((0 < cseqtmp) && (cseqtmp < cseq_counter)) {
+				if ((0 < cseqtmp) && (cseqtmp < msg_data.cseq_counter)) {
 					if (verbose>0) {
 						printf("ignoring retransmission\n");
 					}
@@ -1014,10 +1089,10 @@ void shoot(char *buf, int buff_size)
 					continue;
 					}
 				else if (regexec(&(regexps.authexp), received, 0, 0, 0) == REG_NOERROR) {
-					if (!username && !auth_username) {
-						if (timing > 0) {
-							timing--;
-							if (timing == 0) {
+					if (!msg_data.username && !options->auth_username) {
+						if (timers.timing > 0) {
+							timers.timing--;
+							if (timers.timing == 0) {
 								if (counters.run == 0) {
 									counters.run++;
 								}
@@ -1025,8 +1100,8 @@ void shoot(char *buf, int buff_size)
 								exit_code(0, __PRETTY_FUNCTION__, NULL);
 							}
 							counters.run++;
-							new_transaction(request, response);
-							delays.retryAfter = timer_t1;
+							msg_data.cseq_counter = new_transaction(request, response);
+							delays.retryAfter = timers.timer_t1;
 							continue;
 						}
 						fprintf(stderr, "%s\nerror: received 40[17] but cannot "
@@ -1036,32 +1111,48 @@ void shoot(char *buf, int buff_size)
 					}
 					/* prevents a strange error */
 					regcomp(&(regexps.authexp), "^SIP/[0-9]\\.[0-9] 40[17] ", REG_EXTENDED|REG_NOSUB|REG_ICASE);
-					insert_auth(request, received);
+					insert_auth(request, received, msg_data.username, options->password,
+              options->auth_username, options->authhash, counters.namebeg,
+              counters.nameend);
 					if (verbose > 2)
 						printf("\nreceived:\n%s\n", received);
-					new_transaction(request, response);
+					msg_data.cseq_counter = new_transaction(request, response);
 					continue;
 				} /* if auth...*/
 				/* lets see if received a redirect */
-				if (redirects == 1 && regexec(&(regexps.redexp), received, 0, 0, 0) == REG_NOERROR) {
-					handle_3xx(&(cdata.adr));
+				if (options->redirects == 1 &&
+            regexec(&(regexps.redexp), received, 0, 0, 0) == REG_NOERROR) {
+					handle_3xx(&(cdata.adr), options->warning_ext, cdata.rport,
+                     options->address, cdata.transport, options->outbound_proxy,
+                     options->domainname
+#ifdef WITH_TLS_TRANSP
+                     , options->ignore_ca_fail
+#endif
+                     );
 				} /* if redircts... */
-				else if (trace == 1) {
-					trace_reply();
+				else if (options->mode == SM_TRACE) {
+					trace_reply(options->regex, counters.namebeg, &timers);
 				} /* if trace ... */
-				else if (usrloc == 1||invite == 1||message == 1) {
-					handle_usrloc();
+				else if (options->mode == SM_USRLOC ||
+                 options->mode == SM_USRLOC_INVITE ||
+                 options->mode == SM_USRLOC_MESSAGE ||
+                 options->mode == SM_INVITE ||
+                 options->mode == SM_MESSAGE) {
+					handle_usrloc(options->regex, counters.namebeg, counters.nameend,
+                        options->rand_rem, msg_data.username,
+                        options->nagios_warn, &timers, msg_data.mes_body,
+                        options->mode);
 				}
-				else if (randtrash == 1) {
-					handle_randtrash();
+				else if (options->mode == SM_RANDTRASH) {
+					handle_randtrash(options->warning_ext, counters.nameend);
 				}
 				else {
-					handle_default();
+					handle_default(options->regex, &timers);
 				} /* redirect, auth, and modes */
 			} /* ret > 0 */
 			else if (ret == -1) { // we did not got anything back, send again
 				/* no re-transmission on reliable transports */
-				if (transport != SIP_UDP_TRANSPORT) {
+				if (cdata.transport != SIP_UDP_TRANSPORT) {
 					cdata.dontsend = 1;
 				}
 				continue;
@@ -1071,7 +1162,9 @@ void shoot(char *buf, int buff_size)
 				continue;
 			}
 			else {
-				if (usrloc == 1) {
+				if (options->mode == SM_USRLOC ||
+            options->mode == SM_USRLOC_INVITE ||
+            options->mode == SM_USRLOC_MESSAGE) {
 					printf("failed\n");
 				}
 				perror("socket error");
@@ -1082,25 +1175,25 @@ void shoot(char *buf, int buff_size)
 			if (counters.send_counter == 1) {
 					memcpy(&(timers.firstsendt), &(timers.sendtime), sizeof(struct timeval));
 			}
-			if (namebeg==nameend) {
+			if (counters.namebeg==counters.nameend) {
 				printf("flood end reached\n");
 				printf("it took %.3f ms seconds to send %i request.\n", 
-						deltaT(&(timers.firstsendt), &(timers.sendtime)), namebeg);
+						deltaT(&(timers.firstsendt), &(timers.sendtime)), counters.namebeg);
 				printf("we sent %f requests per second.\n", 
-						(namebeg/(deltaT(&(timers.firstsendt), &(timers.sendtime)))*1000));
+						(counters.namebeg/(deltaT(&(timers.firstsendt), &(timers.sendtime)))*1000));
 				exit_code(0, __PRETTY_FUNCTION__, NULL);
 			}
-			namebeg++;
-			cseq_counter++;
-			create_msg(REQ_FLOOD, request, NULL, usern, cseq_counter);
+			counters.namebeg++;
+			msg_data.cseq_counter++;
+			create_msg(REQ_FLOOD, &msg_data);
 		}
 	} /* while 1 */
 
 	/* this should never happen any more... */
-	if (randtrash == 1) {
+	if (options->mode == SM_RANDTRASH) {
 		exit_code(0, __PRETTY_FUNCTION__, NULL);
 	}
-	printf("** give up retransmissioning....\n");
+	printf("** give up further retransmissions....\n");
 	if (counters.retrans_r_c>0 && (verbose > 1)) {
 		printf("%i retransmissions received during test\n", counters.retrans_r_c);
 	}
